/*
    Copyright (c) 2007, Ben

    All rights reserved.
    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
    *   Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer 
	    in the documentation and/or other materials provided with the distribution.
    *   The names of its contributors may be used to endorse or promote products derived from this software without specific prior 
	    written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
    EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*
 * Implementation of the ServerInfo class.
 */

#include <sstream>
using namespace std;

#include "ServerInfo.h"
#include "stringutils.h"

/*
 * Creates a new instance of the class initializing the name, address and port.
 */
ServerInfo::ServerInfo(string name, string address, int port) {
	this->name = name;
	this->address = address;
	this->port = port;
}

/*
 * Creates a new instance of the class, parsing the line and initializing the values.
 * Used for reading ServerInfo elements in a file.
 */
ServerInfo::ServerInfo(string line) {
	// name;address;port;connection_count;last_on (last on to be done later)
	
	vector<string> tokens;
	Tokenize(line, tokens, ";");
	
	if (tokens.size() == 4) {
		this->name = tokens.at(0);
		this->address = tokens.at(1);
	
		stringstream ss(tokens.at(2));
		ss >> this->port;
	
		stringstream s2(tokens.at(3));
		s2 >> this->connectionCount;
	}
}

/*
 * Returns the name of the server
 */
string ServerInfo::getName() const {
	return this->name;
}

/*
 * Sets the server name
 */
void ServerInfo::setName(string name) {
	this->name = name;
}

/*
 * Returns the address
 */
string ServerInfo::getAddress() const {
	return this->address;
}

/*
 * Sets the server address
 */
void ServerInfo::setAddress(string address) {
	this->address = address;
}

/*
 * Returns the port
 */
const int ServerInfo::getPort() const {
	return this->port;
}

/*
 * Sets the server port
 */
void ServerInfo::setPort(int port) {
	this->port = port;
}

/*
 * Returns the connection count
 */
int ServerInfo::getConnectionCount() const {
	return this->connectionCount;
}

/*
 * Sets the connection count
 */
void ServerInfo::setConnectionCount(int connectionCount) {
	this->connectionCount = connectionCount;
}

/*
 * Returns a string representation used when saving ServerInfo elements in files.
 */
string ServerInfo::serialize() {
	// turn it into a parseable form
	string line(this->name);
	line += ";";
	line += this->address;
	line += ";";
	
	stringstream ss;
	ss << this->port;
	
	line += ss.str();
	line += ";";
	stringstream s2;
	s2 << this->connectionCount;
	
	line += s2.str();
	
	return line;
}

