/*
    Copyright (c) 2007, Ben

    All rights reserved.
    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
    *   Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer 
	    in the documentation and/or other materials provided with the distribution.
    *   The names of its contributors may be used to endorse or promote products derived from this software without specific prior 
	    written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
    EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*
 *  Provies some helper functions useful when loading data from or saving data to files.
 */

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
using namespace std;

#include "fileutils.h"
#include "stringutils.h"

/*
 * Loads ServerInfo elements from a file.
 */
vector<ServerInfo> loadServerInfoFile(string filename) {
	vector<ServerInfo> servers;
	
	ifstream input(filename.c_str());
		
	string line;
	
	while(getline(input, line)) {
		ServerInfo info(line);
		
		servers.push_back(info);
	}
	
	input.close();
	return servers;
}

/*
 * Saves a vector of ServerInfo elements to a file
 */
void saveServerInfoFile(string filename, vector<ServerInfo> servers) {
	ofstream output(filename.c_str());
	
	for(int i=0;i < servers.size(); i++)
	{
		ServerInfo info = servers.at(i);
		output << info.serialize() << endl;
	}
	output.close();
}

/*
 * Loads servers from a cleaned up IBBS text file. This file is available from the internet.
 */
vector<ServerInfo> loadIBBSReport(string filename) {
	vector<ServerInfo> servers;
	
	ifstream input(filename.c_str());
	
	string line;
	
	while(getline(input, line)) {
		string name = line.substr(2,42);
		name = trim(name," ");
		//cout << name << "|";
			
		string address = line.substr(44);
		string port_string = "23";
		int port_index = address.find(":");
		if (port_index < string::npos) {
			port_string = address.substr(port_index+1);
			address = address.substr(0,port_index);
			//cout << address << endl;
		}
		
		//cout << address << "|";
		int port = 23;
		stringstream ss(port_string);
		ss >> port;
		
		//cout << port << endl;
		ServerInfo info(name, address, port);
		info.setConnectionCount(0);
		
		servers.push_back(info);
	}	
	
	input.close();
	return servers;
}

/*
 * Converts a cleaned up IBBS file to our type of input file.
 */
void convertIBBSReport(string source_filename, string dest_filename) {
	vector<ServerInfo> servers = loadIBBSReport(source_filename);
	saveServerInfoFile(dest_filename, servers);	
}
