/*
    Copyright (c) 2007, Ben

    All rights reserved.
    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
    *   Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer 
	    in the documentation and/or other materials provided with the distribution.
    *   The names of its contributors may be used to endorse or promote products derived from this software without specific prior 
	    written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
    EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/
#include <iostream>
using namespace std;

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <time.h>

#include "ServerInfo.h"
#include "fileutils.h"

#define CALL_FAILED -1
#define CALL_UNANSWERED 0
#define CALL_ANSWERED 1

/*
 * Places a "call" to the server defined in the ServerInfo object. Will return a call result.
 * If not using bsd, change the bzero call to memset.
 */
int call(ServerInfo serverInfo)
{	
	int socket_id = socket(AF_INET, SOCK_STREAM, 0);
	
	if (socket_id < 0) {
		cout << "could not open socket.";
		return CALL_FAILED;
	}
	else {
		cout << ".";
		cout.flush();
	}
	
	struct hostent *hp, *gethostbyaddr();
	
	// resolve host by name
	hp = gethostbyname(serverInfo.getAddress().c_str());
	
	if (hp == NULL) {
		cout << "could not resolve host name";
		close(socket_id);
		return CALL_UNANSWERED;
	}
	else {
		cout << ".";
		cout.flush();
	}
	
	struct sockaddr_in myname;
	
	myname.sin_port = htons(serverInfo.getPort());
	myname.sin_family = AF_INET;
	
	bcopy(hp->h_addr_list[0], &myname.sin_addr.s_addr, hp->h_length);
	
	if (connect(socket_id, (struct sockaddr *)&myname, sizeof(myname)) < 0)
	{
		cout << "network connect error";
		close(socket_id);
		return CALL_UNANSWERED;
	}
	else {
		cout << ".";
		cout.flush();
	}
	
	
	char socket_buffer[1024];
	bzero (socket_buffer, sizeof(socket_buffer));
    int read_count = read(socket_id, socket_buffer, sizeof(socket_buffer));
	
	if ( read_count < 0) {
		cout << "socket read error";
		close(socket_id);
		return CALL_FAILED;
	}
	else {
		cout << ".";
		cout.flush();
	}
	
	cout << "answered";
	close(socket_id);
	
	return CALL_ANSWERED;
}

/*
 * Mostly self explanatory.
 */
int main (int argc, char * const argv[]) {
	cout << "Telnet BBS war dialer\n"
	     << "version 1.0\n" << endl;
		 
    if (argc != 2) {
		cout << "Usage:\twardialer filename\n"
		     << "\tExample: wardialer testfile.txt" << endl;
	    return 0;
	}
	
	cout << "Using file: " << argv[1] << endl;

	time_t rawtime_start;
	struct tm * timeinfo_start;

	time ( &rawtime_start );
	timeinfo_start = localtime ( &rawtime_start );
	cout << "Process started on: " << asctime(timeinfo_start) << endl;
	
	vector<ServerInfo> servers;
	servers = loadServerInfoFile(argv[1]);
	
	vector<ServerInfo> processed_servers;
	
	for(int i=0;i < servers.size(); i++)
	{
		ServerInfo info = servers.at(i);
		cout 
		<< "Server Name:      " << info.getName() << "\n"
		<< "Address:          " << info.getAddress() << "\n"
		<< "Port:             " << info.getPort() << "\n"
		<< "Connection Count: " << info.getConnectionCount() << "\n"
		<< "Calling.";
		cout.flush();
		int result = call(info);
		cout << endl;
		cout << "--------------------------------" << endl;
		if (result == CALL_ANSWERED)
			info.setConnectionCount(info.getConnectionCount() + 1);
		processed_servers.push_back(info);
	}
	
	saveServerInfoFile(argv[1], processed_servers);
	
	time_t rawtime_stop;
	struct tm * timeinfo_stop;

	time ( &rawtime_stop );
	timeinfo_stop = localtime ( &rawtime_stop );
	cout << "Process stopped on: " << asctime(timeinfo_stop) << endl;
	
    cout << "Halt\n";
	
	// convertIBBSReport("input.txt", "converted.txt"); 
    return 0;
}
